@@ -1,6 +1,6 @@
-// $Id: bender_quad.cpp 3757 2015-11-30 12:03:07Z bradbell $
+// $Id: bender_quad.cpp 3788 2016-02-09 15:50:06Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-15 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the
@@ -59,7 +59,7 @@ y - [ \partial_y \partial_y F(x, y) ]^{-1} \partial_y F[x,  y]
 
 
 $code
-$verbatim%example/bender_quad.cpp%0%// BEGIN C++%// END C++%1%$$
+$srcfile%example/bender_quad.cpp%0%// BEGIN C++%// END C++%1%$$
 $$
 
 $end
