@@ -1,8 +1,8 @@
-// $Id: lu_vec_ad.hpp 3757 2015-11-30 12:03:07Z bradbell $
-# ifndef CPPAD_LU_VEC_AD_HPP
-# define CPPAD_LU_VEC_AD_HPP
+// $Id: lu_vec_ad.hpp 3804 2016-03-20 15:08:46Z bradbell $
+# ifndef CPPAD_EXAMPLE_LU_VEC_AD_HPP
+# define CPPAD_EXAMPLE_LU_VEC_AD_HPP
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-15 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the
