@@ -1,6 +1,6 @@
-// $Id: numeric_type.cpp 3757 2015-11-30 12:03:07Z bradbell $
+// $Id: numeric_type.cpp 3788 2016-02-09 15:50:06Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-15 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the
@@ -17,7 +17,7 @@ $section The NumericType: Example and Test$$
 $mindex NumericType$$
 
 $code
-$verbatim%example/numeric_type.cpp%0%// BEGIN C++%// END C++%1%$$
+$srcfile%example/numeric_type.cpp%0%// BEGIN C++%// END C++%1%$$
 $$
 
 $end
