@@ -1,6 +1,5 @@
-// $Id: optimize.cpp 3856 2016-12-21 05:51:22Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-17 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the
@@ -424,7 +423,7 @@ namespace {
 
 		// unary operator where operand is arg[0]
 		// (note that sin corresponds to two tape variables)
-		not_used = CppAD::abs(x[0]);
+		not_used = fabs(x[0]);
 		y[0]     = sin(x[0]);
 		original += 3;
 		opt      += 2;
@@ -987,7 +986,7 @@ namespace {
 		);
 		Y[0] = X[0];
 		for(j = 0; j < n_operations; j++)
-			Y[0] = abs(Y[0]);
+			Y[0] = fabs(Y[0]);
 
 		// create f: X -> Y and stop tape recording
 		CppAD::ADFun<double> F;
@@ -1878,6 +1877,101 @@ namespace {
 
 		return ok;
 	}
+
+	// Test case where a variable is removed during optimization
+	// (bug fixed 2017-03-04)
+	bool cond_exp_skip_remove_var(void)
+	{	bool ok = true;
+		using CppAD::vector;
+		using CppAD::AD;
+		using CppAD::NearEqual;
+		double eps10 = 10.0 * std::numeric_limits<double>::epsilon();
+
+		vector< AD<double> > ax(2), ay(2);
+		ax[0] = 1.0;
+		ax[1] = 2.0;
+		Independent(ax);
+		//
+		AD<double> var_1   = ax[0] + ax[1];
+		AD<double> var_2   = ax[0] + ax[1]; // gets removed during optimization
+		AD<double> var_3   = ax[0] + ax[1]; // gets removed during optimization
+		AD<double> var_4   = ax[0] - ax[1];
+		AD<double> par_1   = 1.0;
+		//
+		// first conditional expression depends on var_1
+		// 6 * x_0 if x_0 + x_1 >= 1.0,  7 * x_1 otherwise
+		ay[0] = CppAD::CondExpGe(var_1, par_1, 6.0 * ax[0], 7.0 * ax[1]);
+		//
+		// second conditional expression depends on var_4
+		// 8 * x_0 if x_0 - x_1 >= x_0 + x_1, 9 * x_1 otherwise
+		ay[1] = CppAD::CondExpGe(var_4, par_1, 8.0 * ax[0], 9.0 * ax[1]);
+		CppAD::ADFun<double> f(ax, ay);
+		//
+		if( conditional_skip_ )
+			f.optimize();
+		else
+			f.optimize("no_conditional_skip");
+
+		// check case where x[0] = 2, x[1] = 4
+		vector<double> x(2), y(2);
+		x[0] = 2.0;
+		x[1] = 4.0;
+		y    = f.Forward(0, x);
+		ok &= NearEqual(y[0], 6.0 * x[0], eps10, eps10);
+		ok &= NearEqual(y[1], 9.0 * x[1], eps10, eps10);
+
+		return ok;
+	}
+
+	// Test case where if_false case is not used by conditional expression
+	// but is use after conditional expression.
+	// (bug fixed 2017-04-02)
+	bool cond_exp_if_false_used_after(void)
+	{	bool ok = true;
+		using CppAD::vector;
+		using CppAD::AD;
+		using CppAD::NearEqual;
+		double eps10 = 10.0 * std::numeric_limits<double>::epsilon();
+
+		vector< AD<double> > ax(2), ay(1);
+		ax[0] = 1.0;
+		ax[1] = 2.0;
+		Independent(ax);
+		//
+		AD<double> left     = ax[0];
+		AD<double> right    = ax[1];
+		AD<double> if_true  = ax[0] + ax[0];
+		AD<double> if_false = ax[1] + ax[1];
+		//
+		AD<double> cexp = CondExpLt(left, right, if_true, if_false);
+		ay[0] = cexp + if_false;
+		CppAD::ADFun<double> f(ax, ay);
+		//
+		if( conditional_skip_ )
+			f.optimize();
+		else
+			f.optimize("no_conditional_skip");
+		//
+		// check case where x[0] < x[1]
+		vector<double> x(2), y(1);
+		x[0] = 2.0;
+		x[1] = 4.0;
+		y    = f.Forward(0, x);
+		ok &= NearEqual(y[0], x[0] + x[0] + x[1] + x[1], eps10, eps10);
+		ok &= f.number_skip() == 0;
+		//
+		// check case where x[0] >= x[1] (if_true is not used)
+		x[0] = 4.0;
+		x[1] = 2.0;
+		y    = f.Forward(0, x);
+		ok &= NearEqual(y[0], x[1] + x[1] + x[1] + x[1], eps10, eps10);
+		if( conditional_skip_ )
+			ok &= f.number_skip() == 1;
+		else
+			ok &= f.number_skip() == 0;
+		//
+		return ok;
+	}
 }
 
 bool optimize(void)
@@ -1950,7 +2044,12 @@ bool optimize(void)
 		// check reverse mode conditional skipping
 		ok     &= cond_exp_reverse();
 		// check case where an expresion needed by both true and false case
-		ok     &=  cond_exp_both_true_and_false();
+		ok     &= cond_exp_both_true_and_false();
+		// check case were a variable in left or right expressions
+		// is removed during the optimization
+		ok     &= cond_exp_skip_remove_var();
+		// check case where an if case is used after the conditional expression
+		ok     &= cond_exp_if_false_used_after();
 	}
 	//
 	CppAD::user_atomic<double>::clear();
